﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ChaosEngine.EngineCore;
using ChaosEngine.GPU.Core;

using SlimDX;
using SlimDX.Windows;
using SlimDX.DXGI;
using SlimDX.Direct3D10;
using X = SlimDX.Direct3D10;
namespace ChaosEngine.GPU.API.DirectX
{
    public class GCDX10 : vGPUCore
    {
        internal RenderForm Form;
        internal SwapChain Chain;
        internal SlimDX.Direct3D10.Device Dev;
        internal SlimDX.DXGI.Factory XFactory;
        internal Texture2D BufBack;
        internal RenderTargetView RTView;
        internal X.InputLayout CurLay;
        public GCDX10() : base("Anthony Wells","Alpha 1","DirectX10.1")
        {
            Form = null;
            Chain = null;
            Dev = null;
        }

        public override vGPUCore SpawnInstance()
        {
            return new GCDX10();
        }

        public override bool InitGpu()
        {

            return true;
        }

        public override bool CreateForm(int x, int y, int w, int h, bool full,string app)
        {

            Form = new RenderForm(app + " - Made with Vivid3D(OSCE) - ALPHA SOFTWARE -");
            var desc = new SwapChainDescription()
            {
                BufferCount = 1,
                ModeDescription = new ModeDescription(Form.ClientSize.Width, Form.ClientSize.Height, new Rational(60, 1), Format.R8G8B8A8_UNorm),
                IsWindowed = full ? false : true,
                OutputHandle = Form.Handle,
                SampleDescription = new SampleDescription(1, 0),
                SwapEffect = SwapEffect.Discard,
                Usage = Usage.RenderTargetOutput
            };

            
            
//            Device.CreateWithSwapChain(null, DriverType.Hardware, DeviceCreationFlags.Debug, desc, out device, out swapChain);
            SlimDX.Direct3D10.Device.CreateWithSwapChain(null, DriverType.Hardware, DeviceCreationFlags.Debug, desc, out Dev, out Chain);

            //Stops Alt+enter from causing fullscreen skrewiness.
            XFactory = Chain.GetParent<SlimDX.DXGI.Factory>();
            XFactory.SetWindowAssociation(Form.Handle, WindowAssociationFlags.IgnoreAll);

            BufBack = Texture2D.FromSwapChain<Texture2D>(Chain, 0);
            RTView = new RenderTargetView(Dev, BufBack);

            Dev.OutputMerger.SetTargets(RTView);
            Dev.Rasterizer.SetViewports(new Viewport(0, 0, Form.ClientSize.Width, Form.ClientSize.Height, 0.0f, 1.0f));
            
            return true;
        }

        public override bool EmbedCore(System.Windows.Forms.Control surface, string app)
        {
            var Form = surface;
            var desc = new SwapChainDescription()
            {
                BufferCount = 1,
                ModeDescription = new ModeDescription(Form.ClientSize.Width, Form.ClientSize.Height, new Rational(60, 1), Format.R8G8B8A8_UNorm),
                IsWindowed = true,
                OutputHandle = surface.Handle,
                SampleDescription = new SampleDescription(1, 0),
                SwapEffect = SwapEffect.Discard,
                Usage = Usage.RenderTargetOutput
            };


  

            //            Device.CreateWithSwapChain(null, DriverType.Hardware, DeviceCreationFlags.Debug, desc, out device, out swapChain);
            SlimDX.Direct3D10.Device.CreateWithSwapChain(null, DriverType.Hardware, DeviceCreationFlags.Debug,  desc, out Dev, out Chain);

            //Stops Alt+enter from causing fullscreen skrewiness.
            XFactory = Chain.GetParent<SlimDX.DXGI.Factory>();
            XFactory.SetWindowAssociation(Form.Handle, WindowAssociationFlags.IgnoreAll);

            BufBack = Texture2D.FromSwapChain<Texture2D>(Chain, 0);
            RTView = new RenderTargetView(Dev, BufBack);

            Dev.OutputMerger.SetTargets(RTView);
            Dev.Rasterizer.SetViewports(new Viewport(0, 0, Form.ClientSize.Width, Form.ClientSize.Height, 0.0f, 1.0f));

            return true;
        }

        public override void EmbedBeginDraw()
        {
            Dev.ClearRenderTargetView(RTView, new Color4(ClearColor));
        }

        public override void EmbedEndDraw()
        {
            Chain.Present(0, PresentFlags.None);
        }

        public override int RunCore(VividApp app)
        {

            app.InitApp();

            MessagePump.Run(Form, () =>
            {

                for (int i = 0; i < UpdateCycles; i++)
                {
                    app.UpdateApp();
                }
                
                Dev.ClearRenderTargetView(RTView, new Color4(ClearColor));
                
                app.DrawApp();

                Chain.Present(0, PresentFlags.None);
            });
            return 1;

        }

     
        public override vDataBuffer CreateVertexBuffer(float[] pos, float[] uv, float[] norm)
        {
            var r = new vDX10DataBuffer();

            /*
            f.Layout = new InputLayout(Dev, pass.Description.Signature, new[] {
                new InputElement("POSITION", 0, Format.R32G32B32A32_Float, 0, 0),
                new InputElement("COLOR", 0, Format.R32G32B32A32_Float, 16, 0) 
            });
            */
            int ni = 0;
            r.Stream = new DataStream(pos.Length * 4 + uv.Length * 4 + norm.Length * 4, true, true);
            for (int v = 0; v < pos.Length; v = v + 3)
            {
                r.Stream.Write<float>(pos[v]);
                r.Stream.Write<float>(uv[v]);
                r.Stream.Write<float>(norm[ni]);
                r.Stream.Write<float>(norm[ni = 1]);
                r.Stream.Write<float>(norm[ni + 2]);
                r.Stream.Write<float>(pos[v + 1]);
                r.Stream.Write<float>(uv[v + 1]);
                r.Stream.Write<float>(norm[ni + 3]);
                r.Stream.Write<float>(norm[ni + 4]);
                r.Stream.Write<float>(norm[ni + 5]);
                r.Stream.Write<float>(pos[v + 2]);
                r.Stream.Write<float>(uv[v + 2]);
                r.Stream.Write<float>(norm[ni + 6]);
                r.Stream.Write<float>(norm[ni + 7]);
                r.Stream.Write<float>(norm[ni + 8]);
                ni = ni + 9;
            }
            r.Stream.Position = 0;

            var vertices = new SlimDX.Direct3D10.Buffer(Dev, r.Stream, new BufferDescription()
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)r.Stream.Length,
                Usage = ResourceUsage.Default
            });
            r.Stream.Dispose();
            r.Stream = null;

            r.Data = vertices;

            r.Count = pos.Length;

            return r;

        }

        public override vDataBuffer CreateIndexBuffer(int[] indices)
        {
            var r = new vDX10DataBuffer();

            /*
            f.Layout = new InputLayout(Dev, pass.Description.Signature, new[] {
                new InputElement("POSITION", 0, Format.R32G32B32A32_Float, 0, 0),
                new InputElement("COLOR", 0, Format.R32G32B32A32_Float, 16, 0) 
            });
            */

            r.Stream = new DataStream(indices.Length * 4, true, true);
            r.Stream.WriteRange<int>(indices);
            r.Stream.Position = 0;

            var indices2 = new SlimDX.Direct3D10.Buffer(Dev, r.Stream, new BufferDescription()
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)r.Stream.Length,
                Usage = ResourceUsage.Default
            });
            r.Stream.Dispose();
            r.Stream = null;

            r.Data = indices2;

            r.Count = indices.Length;

            return r;
        }

        public override vEffect EffectFromFile(string addr)
        {
            var path = FSHelper.AddrToPath(addr);
            var fx = new vDX10Effect();
            fx.FX = Effect.FromFile(Dev, path, "fx_4_0", ShaderFlags.OptimizationLevel3, EffectFlags.AllowSlowOperations,null,null);
            return fx;
        }
        public override void EffectSetFloat(vEffect fx, string name, float v) { }
        public override void EffectSetMatrix(vEffect fx, string name, Matrix v) { }

        public override void EffectSetVector3(vEffect fx, string name, Vector3 v) { }

        public override void EffectSetVector4(vEffect fx, string name, Vector4 v) { }

      
        public override void DrawGeo(vDataBuffer vertices, vDataBuffer indices, vEffect fx, int tech, int pass)
        {
            var t = ((vDX10Effect)fx).FX.GetTechniqueByIndex(tech);
            var p = t.GetPassByIndex(pass);
            var layout = new InputLayout(Dev, p.Description.Signature, new[] {
                new InputElement("POSITION", 0, Format.R32G32B32A32_Float, 0, 0),
                new InputElement("TEXCOORD0", 0, Format.R32G32B32A32_Float, 16, 0),
                new InputElement("NORMAL",0,Format.R32G32B32A32_Float,32,0),
                new InputElement("TANGENT",0,Format.R32G32B32A32_Float,48,0),
                new InputElement("BINORMAL",0,Format.R32G32B32A32_Float,64,0)
            });
            CurLay = layout;

            Dev.InputAssembler.SetInputLayout(CurLay);
            Dev.InputAssembler.SetPrimitiveTopology(PrimitiveTopology.TriangleList);
            Dev.InputAssembler.SetVertexBuffers(0,new VertexBufferBinding( ((vDX10DataBuffer)vertices).Data,80,0));
            Dev.InputAssembler.SetIndexBuffer( ((vDX10DataBuffer)indices).Data,Format.R32_UInt,0);

            p.Apply();
            Dev.DrawIndexed(((vDX10DataBuffer)indices).Count,0,0);


        }

    }

    public class vDX10TechniquePass : vTechniquePass{
        public X.EffectPass Pass{get;set;}
    }

    public class vDX10EffectTechnique : vEffectTechnique
    {

        public X.EffectTechnique Tech { get; set; }
    
    }

    public class vDX10Effect : vEffect
    {

        public X.Effect FX { get; set; }
    
    }



    public class vDX10DataBuffer : vDataBuffer 
    {
        public DataStream Stream{get;set;}
        public X.Buffer Data{get;set;}
        public int Count { get; set; }
        public override void CreateBuffer()
        {
            throw new NotImplementedException();
        }
        public override void SetData(float[] data)
        {
            throw new NotImplementedException();
        }

        public override void FreeData()
        {

            if (Stream != null)
            {

                Stream.Dispose();
                Stream = null;

            }

            if (Data != null)
            {

                Data.Dispose();
                Data = null;

            }


        }

    }

}
